@@ -1,12 +1,17 @@
 package ClassLoader;
 
+use strict;
+use warnings;
+
+our $VERSION = 1.07;
+
 # -----------------------------------------------------------------------------
 
 =encoding utf8
 
 =head1 NAME
 
-ClassLoader - Load classes automatically
+ClassLoader - Load Perl Classes Automatically
 
 =head1 SYNOPSIS
 
@@ -16,8 +21,6 @@ ClassLoader - Load classes automatically
 
 =head1 DESCRIPTION
 
-(I<This module is documented in German>)
-
 =head2 Zweck
 
 Müde, C<use>-Anweisungen für das Laden von Perl-Klassen zu schreiben?
@@ -27,8 +30,8 @@ einzige Anweisung:
 
     use ClassLoader;
 
-Danach wird jede Klasse automatisch mit ihrem ersten
-Methodenaufruf geladen. Dies geschieht bei I<jeder> Methode,
+Danach werden alle Klassen automatisch mit ihrem ersten
+Methodenaufruf geladen. Dies geschieht bei jeder Methode,
 gleichgültig, ob Klassen- oder Objektmethode.
 
 =head2 Vorteile
@@ -42,36 +45,37 @@ Man muss keine C<use>-Aufrufe mehr schreiben
 =item *
 
 Es werden nur die Klassen geladen, die das Programm tatsächlich
-benötigt
+nutzt
 
 =item *
 
-Die Startzeit des Programms verkürzt sich
+Die Startzeit des Programms verkürzt sich, da später benötigte
+Klassen erst später geladen werden
 
 =item *
 
-Das Programm benötigt unter Umständen weniger Speicher, da keine
-überflüssigen Module geladen werden
+Das Programm benötigt unter Umständen weniger Speicher, da Klassen,
+die nicht genutzt werden, auch nicht geladen werden
 
 =back
 
 =head2 Was ist ein Klassen-Modul?
 
-Unter einem Klassen-Modul verstehen wir eine pm-Datei, die
-gmäß Perl-Konventionen eine Klasse definiert, d.h. die
+Unter einem Klassen-Modul verstehen wir eine .pm-Datei, die
+gemäß Perl-Konventionen eine Klasse definiert, d.h. die
 
 =over 4
 
-=item 1
+=item 1.
 
 ein Package mit dem Namen der Klasse deklariert,
 
-=item 2
+=item 2.
 
 unter dem Namen des Package gemäß den Perl-Konventionen im
 Dateisystem abgelegt ist,
 
-=item 3
+=item 3.
 
 ihre Basisklassen (sofern vorhanden) selbständig lädt.
 
@@ -79,35 +83,38 @@ ihre Basisklassen (sofern vorhanden) selbständig lädt.
 
 =head2 Beispiel
 
-Eine Klasse My::Class wird in einer Datei mit dem Pfad My/Class.pm
+Eine Klasse I<My::Class> wird in einer Datei mit dem Pfad C<My/Class.pm>
 definiert und irgendwo unter C<@INC> installiert. Sie hat den Inhalt:
 
     package My::Class;
     use base qw/<BASECLASSES>/;
     
-    <METHODS>
+    <SOURCE>
     
     1;
 
-Das Laden der Basisklassen-Module geschieht hier mittels
-C<use base>. Es ist genauso möglich, die Basisklassen-Module per
-C<use> zu laden und C<@ISA> zuzuweisen, was aber umständlicher ist.
+Hierbei ist <BASECLASSES> die Liste der Basisklassen und <SOURCE>
+der Quelltext der Klasse (einschließlich der
+Methodendefinitionen). Das Laden der Basisklassen-Module geschieht
+hier mittels C<use base>. Es ist genauso möglich, die
+Basisklassen-Module per C<use parent> oder direkt per C<use> zu laden
+und ihre Namen C<@ISA> zuzuweisen, was aber umständlicher ist.
 
-Eine pm-Datei, die diesen Konventionen genügt, ist ein
-Klassen-Modul und wird von ClassLoader automatisch beim ersten
+Eine .pm-Datei, die diesen Konventionen genügt, ist ein
+Klassen-Modul und wird von I<< ClassLoader >> automatisch beim ersten
 Methodenzugriff geladen.
 
 =head2 Wie funktioniert das?
 
-ClassLoader installiert sich als Basisklasse von UNIVERSAL und
-definiert eine Methode AUTOLOAD, bei der sämtliche
+I<< ClassLoader >> installiert sich als Basisklasse von I<UNIVERSAL> und
+definiert eine Methode C<AUTOLOAD>, bei der sämtliche
 Methodenaufrufe ankommen, die vom Perl-Interpreter nicht aufgelöst
 werden können. Die AUTOLOAD-Methode lädt das benötigte
 Klassen-Modul und ruft die betreffende Methode auf. Existiert das
 Klassen-Modul nicht oder enthält es die gerufene Methode nicht, wird
 eine Exception ausgelöst.
 
-Die AUTOLOAD-Methode, die ClassLoader definiert, ist recht einfach
+Die AUTOLOAD-Methode, die I<< ClassLoader >> definiert, ist recht einfach
 (Fehlerbehandlung hier vereinfacht):
 
     sub AUTOLOAD {
@@ -129,7 +136,7 @@ Die AUTOLOAD-Methode, die ClassLoader definiert, ist recht einfach
         return $this->$sub(@_);
     }
 
-Lediglich der erste Methodenaufruf einer (noch nicht geladenen)
+Lediglich der erste Methodenaufruf einer noch nicht geladenen
 Klasse läuft über diese AUTOLOAD-Methode. Alle folgenden
 Methodenaufrufe der Klasse finden I<direkt> statt, also ohne
 Overhead! Methodenaufrufe einer explizit geladenen Klasse laufen
@@ -154,27 +161,22 @@ Aufbau des Exception-Texts:
         <METHOD>()
     Error:
         <ERROR>
-    Stack:
+    Stacktrace:
         <STACKTRACE>
 
 =head2 Kann eine Klasse selbst eine AUTOLOAD-Methode haben?
 
-Ja, denn die AUTOLOAD-Methode von ClassLoader wird I<vor> dem Laden
+Ja, denn die AUTOLOAD-Methode von I<< ClassLoader >> wird I<vor> dem Laden
 der Klasse angesprochen. Alle späteren Methoden-Aufrufe der Klasse
 werden über die Klasse selbst aufgelöst. Wenn die Klasse eine
 AUTOLOAD-Methode besitzt, funktioniert diese genau so wie ohne
-ClassLoader.
+I<< ClassLoader >>.
 
 =cut
 
 # -----------------------------------------------------------------------------
 
-use strict;
-use warnings;
-use utf8;
-
-our $VERSION = 1.000060;
-
+# Klasse als Basisklasse von UNIVERSAL installieren
 unshift @UNIVERSAL::ISA,'ClassLoader';
 
 # -----------------------------------------------------------------------------
@@ -183,28 +185,36 @@ unshift @UNIVERSAL::ISA,'ClassLoader';
 
 =head2 AUTOLOAD() - Lade Klassen-Modul
 
+=head3 Synopsis
+
     $this->AUTOLOAD;
 
-Lade Klassen-Modul und führe den Methodenaufruf durch.
-Die Argumente und der Returnwert entsprechen denen der gerufenen Methode.
-Schlägt das Laden des Moduls fehl, löst die Methode eine Exception aus
-(siehe oben).
+=head3 Description
+
+Die Methode lädt fehlende Klassen-Module und führt ihren ersten
+Methodenaufruf durch. Die Argumente und der Returnwert entsprechen
+denen der gerufenen Methode. Schlägt das Laden des Klassen-Moduls fehl,
+löst die Methode eine Exception aus (siehe oben).
 
 Die AUTOLOAD-Methode implementiert die Funktionalität des
-Moduls. Sie wird nicht direkt, sondern vom Perl-Interpreter
-gerufen, wenn eine Methode nicht gefunden wird.
+Moduls ClassLoader. Sie wird nicht direkt, sondern vom
+Perl-Interpreter gerufen, wenn eine Methode nicht gefunden wird.
 
 =cut
 
 # -----------------------------------------------------------------------------
 
+# Interne Hilfsmethode für Exception-Generierung
+
 my $die = sub {
     my ($class,$sub,$error,$msg) = @_;
 
+    # Generiere Stacktrace
+
     my @frames;
     my $i = 1; 
     while (my (undef,$file,$line,$sub) = caller $i++) {
-        $file =~ s|.*/||;
+        # $file =~ s|.*/||;
         push @frames,[$file,$line,$sub];
     }
 
@@ -216,16 +226,20 @@ my $die = sub {
         $stack .= sprintf "%s%s [%s:%s]\n",('  'x$i++),$sub,$file,$line;
     }
     chomp $stack;
-    $stack .= " <== ERROR";
+    # $stack .= " <== ERROR"; # markiere letzten Stackframe mit Fehlerhinweis
     $stack =~ s/^/    /gm;
 
+    # Generiere Meldung
+
     my $str = "Exception:\n    $msg\n";
     $str .= "Class:\n    $class\n";
     $str .= "Method:\n    $sub()\n";
     if ($error) {
         $str .= "Error:\n    $error\n";
     }
-    $str .= "Stack:\n$stack\n";
+    $str .= "Stacktrace:\n$stack\n";
+
+    # Wirf Exception
 
     die $str;
 };
@@ -235,6 +249,9 @@ sub AUTOLOAD {
     # @_: Methodenargumente
 
     my ($class,$sub) = our $AUTOLOAD =~ /^(.*)::(\w+)$/;
+    if (!defined $sub) {
+        warn "OOPS: AUTOLOAD=$AUTOLOAD\n";
+    }
     return if $sub !~ /[^A-Z]/;
 
     eval "use $class";
@@ -261,8 +278,8 @@ sub AUTOLOAD {
 =item *
 
 Der Mechanismus funktioniert nicht, wenn der Modulpfad anders
-lautet als die Klasse heißt. Solche Module müssen explizit per
-use geladen werden.
+lautet als die Klasse heißt. Solche Module müssen explizit
+per use geladen werden.
 
 =item *
 
@@ -272,40 +289,39 @@ erfolgen. Am besten lädt man solche Module auch explizit.
 
 =item *
 
-Über Aufruf der Methode import() ist es nicht möglich, ein
+Über Aufruf der Methode C<import()> ist es nicht möglich, ein
 Modul automatisch zu laden, da Perl bei Nichtexistenz von
-import() AUTOLOAD() nicht aufruft, sondern den Aufruf
-ignoriert. Man kann durch $class->import() also nicht
+C<import()> C<AUTOLOAD()> nicht aufruft, sondern den Aufruf
+ignoriert. Man kann durch C<< $class->import() >> also nicht
 das Laden eines Klassen-Moduls auslösen.
 
 =item *
 
 Module, die nicht objektorientiert, sondern Funktionssammlungen
-sind, werden von ClassLoader nicht behandelt. Diese müssen
-per use geladen werden.
+sind, werden von I<< ClassLoader >> nicht behandelt. Diese sollten
+per C<use> geladen werden. Es gibt im Perl-Core ein Pragma C<autouse>,
+das alternativ zum automatischen Laden von Funktionen verwendet
+werden kann.
 
 =back
 
-=head1 EXCEPTIONS
+=head1 VERSION
 
-    CLASSLOADER-00001: Modul kann nicht geladen werden
-    CLASSLOADER-00002: Methode existiert nicht
+1.07
 
 =head1 AUTHOR
 
-Frank Seitz, http://www.fseitz.de/
-
-=head1 COPYRIGHT
-
-Copyright (C) Frank Seitz, 2008-2010
+Frank Seitz, L<http://fseitz.de/>
 
 =head1 LICENSE
 
-This library is free software; you can redistribute it and/or modify
+This code is free software. You can redistribute it and/or modify
 it under the same terms as Perl itself.
 
 =cut
 
+# -----------------------------------------------------------------------------
+
 1;
 
 # eof
